from importlib.metadata import version

from bboxconverter.io.api import (
    # Input (readers)
    read_coco,
    read_csv,
    read_manifest,
    read_xml,
)

__version__ = version("bboxconverter")

__doc__ = """
bboxconverter - easily convert bounding boxes in different formats
=====================================================================

Main Features
-------------
Here are just a few of the things that bboxconverter does well:

- Importing PASCAL VOC XML files
- Importing YOLO TXT files
- Importing COCO JSON files
- Importing CSV files
- Importing TXT files
- Exporting PASCAL VOC XML files
- Exporting YOLO TXT files
- Exporting COCO JSON files
- Exporting CSV files

"""



from pathlib import Path


# class representing a Custom Label JSON line for an image
class cl_json_line:

    def __init__(self, job, img, img_path):

        #Get image info. Annotations are dealt with seperately
        sizes = []
        image_size = {}
        image_size["width"] = img["width"]
        image_size["depth"] = 3
        image_size["height"] = img["height"]
        sizes.append(image_size)

        bounding_box = {}
        bounding_box["annotations"] = []
        bounding_box["image_size"] = sizes

        self.__dict__["source-ref"] = s3_path + img_path + img['file_name']
        self.__dict__[job] = bounding_box

        #get metadata
        metadata = {}
        metadata['job-name'] = job_name
        metadata['class-map'] = {}
        metadata['human-annotated'] = 'yes'
        metadata['objects'] = []
        metadata['creation-date'] = img['file_name']
        metadata['type'] = 'groundtruth/object-detection'

        self.__dict__[job + '-metadata'] = metadata


print("Getting image, annotations, and categories from COCO file...")


def coco_to_manifest(manifest_file, coco_json_file, path_to_img):
    # Create empty manifest file
    open(manifest_file, 'w').close()

    with open(coco_json_file) as f:
        #Get custom label compatible info
        js = json.load(f)
        images = js['images']
        categories = js['categories']
        annotations = js['annotations']

        print('Images: ' + str(len(images)))
        print('annotations: ' + str(len(annotations)))
        print('categories: ' + str(len(categories)))

    print("Creating CL JSON lines...")
    images_dict = {
        image['id']: cl_json_line(label_attribute, image, path_to_img)
        for image in images
    }

    print('Parsing annotations...')
    for annotation in annotations:
        image = images_dict[annotation['image_id']]
        cl_annotation = {}
        cl_class_map = {}

        # get bounding box information
        cl_bounding_box = {}
        cl_bounding_box['left'] = annotation['bbox'][0]
        cl_bounding_box['top'] = annotation['bbox'][1]

        cl_bounding_box['width'] = annotation['bbox'][2]
        cl_bounding_box['height'] = annotation['bbox'][3]
        cl_bounding_box['class_id'] = annotation['category_id']

        getattr(image, label_attribute)['annotations'].append(cl_bounding_box)

        for category in categories:
            if annotation['category_id'] == category['id']:
                getattr(image, label_attribute + '-metadata')['class-map'][
                    category['id']] = category['name']

        cl_object = {}
        cl_object['confidence'] = int(1)  #not currently used by Custom Labels
        getattr(image,
                label_attribute + '-metadata')['objects'].append(cl_object)
    print('Done parsing annotations')

    # Create manifest file.
    print('Writing Custom Labels manifest...')
    for im in images_dict.values():
        with open(manifest_file, 'a+') as outfile:
            json.dump(im.__dict__, outfile)
            outfile.write('\n')
            outfile.close()



from ..core.bbox_parser import BboxParser
from pandas import DataFrame, json_normalize, concat
import json
from pathlib import Path


def get_bbox_type(df) -> str:
    '''
    Get the bounding box type of a DataFrame.
    '''
    bbox_types = [('x_center', 'y_center', 'width', 'height'),
                  ('x_max', 'y_max', 'x_min', 'y_min'),
                  ('x_min', 'y_min', 'width', 'height')]
    for bbox_type, cols in enumerate(bbox_types):
        if all(col in df.columns for col in cols):
            return ['cwh', 'tlbr', 'tlwh'][bbox_type]
    return None


def read_manifest(path: "str | Path", configuration, format='auto') -> BboxParser:
    '''
    Read bounding boxes from a manifest file using pandas.read_csv.

    Parameters
    ----------
    path : str | Path
        Path to csv file
    format : str
        Format of the manifest file. Can be one of the following: 'auto', 'coco', 'pascal'
    configuration : dict
        Dictionary containing the configuration of the manifest file.
    '''

    mapping = {
        'top': 'y_min',
        'left': 'x_min',
        'right': 'x_max',
        'bottom': 'y_max',
        'width': 'width',
        'height': 'height',
        'class': 'class_name',
        'confidence': 'confidence',
        'image_width': 'image_width',
        'image_height': 'image_height',
        'image_channels': 'image_channel',
        'file_path': 'file_path'
    }

    with open(path, 'r') as f:

        data = DataFrame()
        for line in f:
            json_obj = json.loads(line)

            # Get the image file path
            file_path = json_obj['source-ref']

            # Get the image information
            img = json_obj[configuration['labelling-job-name']]['image_size'][0]
            image_width, image_height, image_channel = img.values()

            # Get the bboxes mapping of the classes
            class_map = json_obj[configuration['labelling-job-metadata']]['class-map']
            class_map = {int(k): v for k, v in class_map.items()}

            # Get the bboxes annotations
            annotations = json_obj[configuration['labelling-job-name']]['annotations']
            bboxes = json_normalize(annotations)

            # Add the image information to the bboxes
            bboxes['image_width'] = image_width
            bboxes['image_height'] = image_height
            bboxes['image_channel'] = image_channel
            bboxes['file_path'] = file_path
            bboxes['class_name'] = bboxes['class_id'].map(class_map)
            bboxes.drop(columns=['class_id'], inplace=True)

            data = concat([data, bboxes], ignore_index=True)

    data.rename(columns=mapping, inplace=True)

    return BboxParser(data, 'tlwh')



class BBox:
    class_name: str = None
    file_path: str = None
    confidence: float = None
    image_width: int = None
    image_height: int = None

    def __init__(self,
                 class_name,
                 file_path,
                 confidence=None,
                 image_width=None,
                 image_height=None) -> None:
        self.class_name = class_name
        self.file_path = file_path
        if confidence != None:
            self.confidence = confidence
        if image_width != None:
            self.image_width = image_width
        if image_height != None:
            self.image_height = image_height
        pass

    def __str__(self) -> str:
        return f"A {self.class_name} detected with {self.confidence} confidence in image {self.file_path} of size {self.image_width} x {self.image_height}"

    def __eq__(self, o: object) -> bool:
        return self.class_name == o.class_name and self.file_path == o.file_path and self.confidence == o.confidence and self.image_width == o.image_width and self.image_height == o.image_height


class TLWH_BBox(BBox):
    x_min: int = None
    y_min: int = None
    width: int = None
    height: int = None

    def __init__(self,
                 class_name,
                 file_path,
                 x_min,
                 y_min,
                 width,
                 height,
                 confidence=None,
                 image_width=None,
                 image_height=None) -> None:
        super().__init__(class_name, file_path, confidence, image_width,
                         image_height)
        self.x_min = x_min
        self.y_min = y_min
        self.width = width
        self.height = height
        pass

    @classmethod
    def from_TLBR(self, bbox) -> None:
        x_min = bbox.x_min
        y_min = bbox.y_min
        width = bbox.x_max - bbox.x_min
        height = bbox.y_max - bbox.y_min
        return self(class_name=bbox.class_name,
                    file_path=bbox.file_path,
                    x_min=x_min,
                    y_min=y_min,
                    width=width,
                    height=height,
                    confidence=bbox.confidence,
                    image_width=bbox.image_width,
                    image_height=bbox.image_height)

    @classmethod
    def from_CWH(self, bbox) -> None:
        width = bbox.width * bbox.image_width
        height = bbox.height * bbox.image_height
        x_min = ((2 * bbox.x_center * bbox.image_width) - width) / 2
        y_min = ((2 * bbox.y_center * bbox.image_height) - height) / 2

        return self(class_name=bbox.class_name,
                    file_path=bbox.file_path,
                    x_min=x_min,
                    y_min=y_min,
                    width=width,
                    height=height,
                    confidence=bbox.confidence,
                    image_width=bbox.image_width,
                    image_height=bbox.image_height)

    def __str__(self) -> str:
        return "TLWH format: " + super().__str__(
        ) + f" {self.x_min} {self.y_min} {self.width} {self.height}"

    def to_dict(self) -> dict:
        return {k: v for k, v in vars(self).items() if v is not None}

    def __eq__(self, o: object) -> bool:
        return super().__eq__(o) and self.x_min == o.x_min and self.y_min == o.y_min and self.width == o.width and self.height == o.height


class TLBR_BBox(BBox):
    x_min: int = None
    y_min: int = None
    x_max: int = None
    y_max: int = None

    def __init__(self,
                 class_name,
                 file_path,
                 x_min,
                 y_min,
                 x_max,
                 y_max,
                 confidence=None,
                 image_width=None,
                 image_height=None) -> None:
        super().__init__(class_name, file_path, confidence, image_width,
                         image_height)
        self.x_min = x_min
        self.y_min = y_min
        self.x_max = x_max
        self.y_max = y_max
        pass

    @classmethod
    def from_TLWH(self, bbox) -> None:
        x_min = bbox.x_min
        y_min = bbox.y_min
        x_max = bbox.x_min + bbox.width
        y_max = bbox.y_min + bbox.height
        return self(class_name=bbox.class_name,
                    file_path=bbox.file_path,
                    x_min=x_min,
                    y_min=y_min,
                    x_max=x_max,
                    y_max=y_max,
                    confidence=bbox.confidence,
                    image_width=bbox.image_width,
                    image_height=bbox.image_height)

    @classmethod
    def from_CWH(self, bbox) -> None:
        w = bbox.width * bbox.image_width
        h = bbox.height * bbox.image_height
        x_min = ((2 * bbox.x_center * bbox.image_width) - w) / 2
        y_min = ((2 * bbox.y_center * bbox.image_height) - h) / 2
        x_max = x_min + w
        y_max = y_min + h
        return self(class_name=bbox.class_name,
                    file_path=bbox.file_path,
                    x_min=x_min,
                    y_min=y_min,
                    x_max=x_max,
                    y_max=y_max,
                    confidence=bbox.confidence,
                    image_width=bbox.image_width,
                    image_height=bbox.image_height)

    def __str__(self) -> str:
        return "TLBR format: " + super().__str__(
        ) + f" {self.x_min} {self.y_min} {self.x_max} {self.y_max}"

    def to_dict(self) -> dict:
        return {k: v for k, v in vars(self).items() if v is not None}

    def __eq__(self, o: object) -> bool:
        return super().__eq__(o) and self.x_min == o.x_min and self.y_min == o.y_min and self.x_max == o.x_max and self.y_max == o.y_max


class CWH_BBox(BBox):
    x_center: int = None
    y_center: int = None
    width: int = None
    height: int = None

    def __init__(self,
                 class_name,
                 file_path,
                 x_center,
                 y_center,
                 width,
                 height,
                 confidence=None,
                 image_width=None,
                 image_height=None) -> None:
        super().__init__(class_name, file_path, confidence, image_width,
                         image_height)
        self.x_center = x_center
        self.y_center = y_center
        self.width = width
        self.height = height
        pass

    @classmethod
    def from_TLBR(self, bbox) -> None:
        x_center = (bbox.x_min + bbox.x_max) / (2 * bbox.image_width)
        y_center = (bbox.y_min + bbox.y_max) / (2 * bbox.image_height)
        width = (bbox.x_max - bbox.x_min) / bbox.image_width
        height = (bbox.y_max - bbox.y_min) / bbox.image_height
        return self(class_name=bbox.class_name,
                    file_path=bbox.file_path,
                    x_center=x_center,
                    y_center=y_center,
                    width=width,
                    height=height,
                    confidence=bbox.confidence,
                    image_width=bbox.image_width,
                    image_height=bbox.image_height)

    @classmethod
    def from_TLWH(self, bbox) -> None:
        x_center = (2*bbox.x_min + bbox.width) / (2*bbox.image_width)
        y_center = (2*bbox.y_min + bbox.height) / (2*bbox.image_height)
        width = bbox.width / bbox.image_width
        height = bbox.height / bbox.image_height
        return self(class_name=bbox.class_name,
                    file_path=bbox.file_path,
                    x_center=x_center,
                    y_center=y_center,
                    width=width,
                    height=height,
                    confidence=bbox.confidence,
                    image_width=bbox.image_width,
                    image_height=bbox.image_height)

    def __str__(self) -> str:
        return "CWH format: " + super().__str__(
        ) + f" {self.x_center} {self.y_center} {self.width} {self.height}"

    def to_dict(self) -> dict:
        return {k: v for k, v in vars(self).items() if v is not None}

    def __eq__(self, o: object) -> bool:
        return super().__eq__(o) and self.x_center == o.x_center and self.y_center == o.y_center and self.width == o.width and self.height == o.height



from ..core.bbox_parser import BboxParser
from pathlib import Path
from json import loads
from pandas.core.frame import DataFrame

def read_coco(path: str | Path):

    with open(path, 'r') as f:
        data = loads(f.read())
        categories = {cat['id']: cat['name'] for cat in data['categories']}
        images = {img['id']: img for img in data['images']}
        df_bbox = DataFrame(data['annotations'])
        df_bbox['class_name'] = df_bbox['category_id'].map(categories)
        df_bbox['file_path'] = df_bbox['image_id'].map(lambda x: images[x]['file_name'])
        df_bbox['image_width'] = df_bbox['image_id'].map(lambda x: images[x]['width'])
        df_bbox['image_height'] = df_bbox['image_id'].map(lambda x: images[x]['height'])
        df_bbox[['x_min','y_min','width','height']] = DataFrame(df_bbox['bbox'].tolist(), index=df_bbox.index)
        df_bbox.drop(columns=['category_id', 'image_id','ignore','iscrowd','area','id','segmentation','bbox'], inplace=True)

    return BboxParser(df_bbox, 'tlwh')







from pathlib import Path
from pandas.core.frame import DataFrame
import json


def to_coco(df_bbox: DataFrame, output_path: "str | Path"):
    '''
    This function will take an array of bounding box and write it to a .json file respecting the coco format

    Parameters
    ----------
    df_bbox : DataFrame
        The dataframe containing the bounding box informations
    output_path : str | Path
        The path to the output file.
    '''

    # Create images
    images = []
    for i, img in enumerate(df_bbox['file_path'].unique()):
        row = df_bbox[df_bbox['file_path'] == img].iloc[0]
        images.append(
            dict(
                width=int(row['image_width']),
                height=int(row['image_height']),
                id=i,
                file_name=img,
            ))

    # Create categories
    categories = []
    for i, cat in enumerate(df_bbox['class_name'].unique()):
        categories.append(dict(id=i, name=cat))

    # Create annotations
    image_id = {img:i for i, img in enumerate(df_bbox['file_path'].unique())}
    cat_id = {cat['name']:cat['id'] for cat in categories}

    annotations = df_bbox.apply(
        lambda row: {
            'id': row.name,
            'image_id': image_id[row['file_path']],
            'category_id': cat_id[row['class_name']],
            'segmentation': [],
            'bbox': [row['x_min'], row['y_min'], row['width'], row['height']],
            'area': row['width'] * row['height'],
            'iscrowd': 0,
            'ignore': 0,
        },
        axis=1).tolist()

    # Create yolo object
    yolo_object = {
        'images': images,
        'categories': categories,
        'annotations': annotations
    }

    # Write to file
    with open(output_path, 'wt', encoding='UTF-8') as outputFile:
        json.dump(yolo_object, outputFile)



from ..core.bbox_parser import BboxParser
from xmltodict import parse
import xml.etree.ElementTree as ET
from pathlib import Path


def read_xml(path: "str | Path", mapping=None, kwargs={}) -> None:

    # parse xml file
    tree = ET.parse("PATH_TO_XML")
    root = tree.getroot()  # get root object

    height = int(root.find("size")[0].text)
    width = int(root.find("size")[1].text)
    channels = int(root.find("size")[2].text)

    with open("XML_PATH") as file:
        file_data = file.read()  # read file contents

        # parse data using package
        dict_data = parse(file_data)
        print("Not Implemented")
        pass



from pandas.core.frame import DataFrame
from sklearn.model_selection import GroupShuffleSplit
from bboxconverter.core.bbox import BBox, TLBR_BBox, TLWH_BBox, CWH_BBox
from bboxconverter.io.writer_coco import to_coco
from bboxconverter.io.writer_yolo import to_yolo
from bboxconverter.io.writer_pascal_voc import to_pascal_voc
from pathlib import Path
from shutil import copy

FORMAT = ['voc', 'coco', 'yolo', 'jsonlines']
TYPES = ['tlbr', 'tlwh', 'cwh']


class BboxParser():
    """
    Bounding box parser class

    Parameters
    ----------
    data : pandas.DataFrame
        Dataframe containing generic bounding boxes. Could contains some of the following columns:
            -'class_name'
            -'file_path'
            -'x_min'
            -'y_min'
            -'x_max'
            -'y_max'
            -'x_center'
            -'y_center'
            -'width'
            -'height'
            -'confidence'
            -'image_height'
            -'image_width'
            -'image_channels'
    bbox_type : str
        Type of bounding box. Can be one of the following: 'tlbr', 'tlwh', 'cwh'
    """

    data: DataFrame = None
    bbox_type: str = None

    def __init__(self, data: DataFrame, bbox_type) -> None:
        self.data = data
        self.bbox_type = bbox_type

    def create_bbox(self, bbox_type: str, **kwargs) -> BBox:
        '''
        Create bounding box object from a dictionary of parameters

        Parameters
        ----------
        bbox_type : str
            Type of bounding box. Can be one of the following: 'tlbr', 'tlwh', 'cwh'
        **kwargs : dict
            Dictionary of parameters for bounding box
        '''
        if bbox_type == 'tlbr':
            return TLBR_BBox(**kwargs)
        if bbox_type == 'tlwh':
            return TLWH_BBox(**kwargs)
        if bbox_type == 'cwh':
            return CWH_BBox(**kwargs)
        return None

    def create_data_splits(self,
                           output_path,
                           train_size=0.8,
                           test_size=0.2,
                           save_func=to_coco):
        # Group split
        splitter = GroupShuffleSplit(
            train_size=train_size,
            test_size=test_size,
            n_splits=1,
            random_state=7
        )
        split = splitter.split(self.data, groups=self.data['file_path'])
        train_inds, test_inds = next(split)
        train = self.data.iloc[train_inds]
        test = self.data.iloc[test_inds]

        # Directory management
        annotation_file_name = Path(output_path).name
        train_folder = Path(output_path).parent / 'train'
        test_folder = Path(output_path).parent / 'test'
        train_image_folder = train_folder / 'images'
        test_image_folder = test_folder / 'images'
        for folder in [
                train_folder, test_folder, train_image_folder,
                test_image_folder
        ]:
            folder.mkdir(parents=True, exist_ok=True)

        # Copy images
        train['file_path'].apply(
            lambda x:
            copy(
                Path(output_path).parent / x,
                train_image_folder / Path(x).name
            )
        )
        test['file_path'].apply(
            lambda x:
            copy(
                Path(output_path).parent / x,
                test_image_folder / Path(x).name
            )
        )

        # Save annotations
        save_func(train, str(train_folder / annotation_file_name))
        save_func(test, str(test_folder / annotation_file_name))

    def export(self,
               output_path: "str | Path",
               format: str,
               split=False,
               train_size=0.8,
               test_size=0.2) -> None:
        '''
        Export bounding boxes to a popular file format:

        - "voc" => Pascal VOC 
        - "coco" => COCO
        - "yolo" => YOLO
        - "jsonlines" => Sagemaker

        If split is False, the output file will contain all bounding boxes. If split is True, the output file will contain the train and test split of the dataset.

        Parameters
        ----------
        output_path : str | Path
            Path to output file. The path should include the file name and extension.
        format : str
            Format of output file. Can be one of the following: 'voc', 'coco', 'yolo', 'sagemaker'
        type : str
            Type of bounding box. Can be one of the following: 'tlbr', 'tlwh', 'cwh'
        split : bool
            Split the dataset into train and test using scikit-learn train_test_split function.
        train_size : float
            If float, should be between 0.0 and 1.0 and represent the proportion of the dataset to include in the test split. If int, represents the absolute number of test samples. If None, the value is set to the complement of the train size.
        test_size : float
            If float, should be between 0.0 and 1.0 and represent the proportion of the dataset to include in the train split. If int, represents the absolute number of train samples. If None, the value is automatically set to the complement of the test size.

        '''
        # Check if bounding box type is set
        type = self.bbox_type
        assert type is not None
        if type not in TYPES:
            raise ValueError(f"Invalid bbox type: {type}")

        # Set export to file function
        save_func = {
            'coco': to_coco,
            'voc': to_pascal_voc,
            'yolo': to_yolo
        }.get(format, None)
        if save_func is None:
            raise ValueError(f"Invalid save function: {format}")

        format_type = {'coco': 'tlwh', 'voc': 'tlbr', 'yolo': 'cwh'}

        # Check if conversion is needed
        if type == format_type[format]:
            if split:
                self.create_data_splits(output_path, train_size, test_size,
                                        save_func)
            else:
                save_func(self.data, output_path)
            return

        # Set conversion function
        format_map = {
            ('voc', 'tlwh'): TLBR_BBox.from_TLWH,
            ('voc', 'cwh'): TLBR_BBox.from_CWH,
            ('coco', 'tlbr'): TLWH_BBox.from_TLBR,
            ('coco', 'cwh'): TLWH_BBox.from_CWH,
            ('yolo', 'tlbr'): CWH_BBox.from_TLBR,
            ('yolo', 'tlwh'): CWH_BBox.from_TLWH,
        }

        # Get conversion function
        convert_func = format_map.get((format.lower(), self.bbox_type))
        if convert_func is None:
            raise ValueError(f"Invalid export format: {format}")

        # Transform data to bounding boxes
        bboxes = self.data.drop(
            columns=['image_channel'], errors='ignore').apply(
                lambda x: self.create_bbox(self.bbox_type, **x.to_dict()),
                axis=1)

        # Serialize bounding boxes
        bboxes = bboxes.apply(lambda x: convert_func(x).to_dict())
        df_bbox = DataFrame.from_records(bboxes)

        # Save to file
        if split:
            self.create_data_splits(output_path, train_size, test_size,
                                    save_func)
        else:
            save_func(df_bbox, output_path)

    def to_csv(self, output_path: "str | Path", type) -> None:
        '''
        Export bounding boxes to a csv file.

        Parameters
        ----------
        output_path : str | Path
            Path to output file
        type : str
            Type of bounding box. Can be one of the following: 'tlbr', 'tlwh', 'cwh'
        '''
        assert self.bbox_type is not None
        if type not in TYPES:
            raise ValueError(f"Invalid bbox type: {type}")

        # Conversion function map (output_type, input_bbox_type)
        # Each type should have two functions to convert from TLBR, TLWH, CWH
        type_map = {
            ('tlbr', 'tlwh'): TLBR_BBox.from_TLWH,
            ('tlbr', 'cwh'): TLBR_BBox.from_CWH,
            ('tlwh', 'tlbr'): TLWH_BBox.from_TLBR,
            ('tlwh', 'cwh'): TLWH_BBox.from_CWH,
            ('cwh', 'tlbr'): CWH_BBox.from_TLBR,
            ('cwh', 'tlwh'): CWH_BBox.from_TLWH,
        }

        if type == self.bbox_type:
            # No conversion needed
            self.data.to_csv(output_path, index=False)
            return

        # Get conversion function
        convert_func = type_map.get((type, self.bbox_type))

        if convert_func is None:
            raise ValueError(f"Invalid bbox type: {type}")

        # Transform data to bounding boxes
        bboxes = self.data.drop(
            columns=['image_channel'], errors='ignore').apply(
                lambda x: self.create_bbox(self.bbox_type, **x.to_dict()),
                axis=1)

        # Serialize bounding boxes
        bboxes = bboxes.apply(lambda x: convert_func(x).to_dict())

        # Save to file
        DataFrame.from_records(bboxes).to_csv(output_path, index=False)

    def __str__(self) -> str:
        return self.data.to_string()



from pandas import read_csv as pd_read_csv
from ..core.bbox_parser import BboxParser
from pathlib import Path


def get_bbox_type(df) -> str:
    bbox_types = [('x_center', 'y_center', 'width', 'height'),
                  ('x_max', 'y_max', 'x_min', 'y_min'),
                  ('x_min', 'y_min', 'width', 'height')]
    for bbox_type, cols in enumerate(bbox_types):
        if all(col in df.columns for col in cols):
            return ['cwh', 'tlbr', 'tlwh'][bbox_type]

    throw_error = f'Could not find bbox type. Columns must have one of the following: {bbox_types}'
    raise SyntaxError(throw_error)


def read_csv(path: "str | Path", mapping=None, kwargs={}):
    '''
    Read bounding boxes from a csv file using pandas.read_csv.

    Parameters
    ----------
    path : str | Path
        Path to csv file
    mapping : dict
        Dictionary to map column names to bboxconverter standard format
    kwargs : dict
        Keyword arguments for pandas.read_csv
    '''
    # Parse data
    bbox_df = pd_read_csv(path, **kwargs)

    # Store them into bbox standard format
    if mapping != None:
        reversed_map = {value: key for key, value in mapping.items()}
        bbox_df.rename(columns=reversed_map, inplace=True)

    # Get bbox type
    bbox_type = get_bbox_type(bbox_df)

    return BboxParser(bbox_df, bbox_type)



from bboxconverter.io.reader_coco import read_coco
from bboxconverter.io.reader_csv import read_csv
from bboxconverter.io.reader_manifest import read_manifest
from bboxconverter.io.reader_xml import read_xml
from bboxconverter.io.writer_coco import to_coco
from bboxconverter.io.writer_pascal_voc import to_pascal_voc
from bboxconverter.io.writer_yolo import to_yolo


__all__ = [
    # Input (readers)
    "read_coco",
    "read_csv",
    "read_manifest",
    "read_xml",
    # Output (writers)
    "to_coco",
    "to_pascal_voc"
    "to_yolo",
]



from pathlib import Path
from pandas.core.frame import DataFrame
import json


def to_yolo(df_bbox: DataFrame, output_path: "str | Path"):
    '''
    This function will take an array of bounding box and write it to a .json file respecting the yolo format

    Parameters
    ----------
    bboxes : DataFrame
        The dataframe containing the bounding box informations
    output_path : str | Path
        The path to the output file
    '''
    print('Not implemented yet')
    return

    images = df_bbox.apply(lambda row: {
        'id': row.name,
        'file_name': row['file_path'],
        'width': row['image_width'],
        'height': row['image_height']
    },
                           axis=1).tolist()

    categories = []
    for i, cat in enumerate(df_bbox['class_name'].unique()):
        categories.append(dict(id=i, name=cat))

    annotations = df_bbox.apply(
        lambda row: {
            'id': row.name,
            'image_id': row.name,
            'category_id': row['class_name'],
            'bbox':
            [row['x_center'], row['y_center'], row['width'], row['height']],
            'area': row['width'] * row['height'],
            'iscrowd': 0
        },
        axis=1).tolist()

    yolo_object = {
        'images': images,
        'categories': categories,
        'annotations': annotations
    }
    with open(output_path, 'w') as outputFile:
        json.dump(yolo_object, outputFile)



from ..core.bbox_parser import BboxParser
import xml.etree.ElementTree as ET
from pathlib import Path


def read_xml(path: "str | Path", mapping=None, kwargs={}) -> None:

    # parse xml file
    tree = ET.parse("PATH_TO_XML")
    root = tree.getroot()  # get root object

    height = int(root.find("size")[0].text)
    width = int(root.find("size")[1].text)
    channels = int(root.find("size")[2].text)

    with open("XML_PATH") as file:
        file_data = file.read()  # read file contents

        # parse data using package
        #dict_data = xmltodict.parse(file_data)
        print("Not Implemented")
        pass


def parse_xml(args):
    xml_path, img_path = args
    tree = ET.parse(xml_path)
    root = tree.getroot()
    size = root.find('size')
    w = int(size.find('width').text)
    h = int(size.find('height').text)
    bboxes = []
    labels = []
    bboxes_ignore = []
    labels_ignore = []
    for obj in root.findall('object'):
        name = obj.find('name').text
        label = label_ids[name]
        difficult = int(obj.find('difficult').text)
        bnd_box = obj.find('bndbox')
        bbox = [
            int(bnd_box.find('xmin').text),
            int(bnd_box.find('ymin').text),
            int(bnd_box.find('xmax').text),
            int(bnd_box.find('ymax').text)
        ]
        if difficult:
            bboxes_ignore.append(bbox)
            labels_ignore.append(label)
        else:
            bboxes.append(bbox)
            labels.append(label)
    if not bboxes:
        bboxes = np.zeros((0, 4))
        labels = np.zeros((0, ))
    else:
        bboxes = np.array(bboxes, ndmin=2) - 1
        labels = np.array(labels)
    if not bboxes_ignore:
        bboxes_ignore = np.zeros((0, 4))
        labels_ignore = np.zeros((0, ))
    else:
        bboxes_ignore = np.array(bboxes_ignore, ndmin=2) - 1
        labels_ignore = np.array(labels_ignore)
    annotation = {
        'filename': img_path,
        'width': w,
        'height': h,
        'ann': {
            'bboxes': bboxes.astype(np.float32),
            'labels': labels.astype(np.int64),
            'bboxes_ignore': bboxes_ignore.astype(np.float32),
            'labels_ignore': labels_ignore.astype(np.int64)
        }
    }
    return annotation